#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "pcx.h"
#include "keys.h"
#include "editskin.h"
#include "config.h"
#include "srl.h"

void parse_error(char *name, long line, char *s)
{
 fprintf(stderr,"line %d: parse error in %s: %s\n",line+1,name,s);
}

int check_boolean(s_line tmp, char *name, int line)
{
 int t;
 
 t=0;
 
 if (tmp.cellcount!=2)
 { 
  if (tmp.cellcount>2)
   fprintf(stderr,"line %d: too many parameters\n",line+1);
  else
   fprintf(stderr,"line %d: too few parameters\n",line+1);
  t=-1;
 } else
 {
  if (strcasecmp(tmp.cell[1],"true")	== 0 ||
      strcasecmp(tmp.cell[1],"1")	== 0 ||
      strcasecmp(tmp.cell[1],"yes")	== 0   )
  {
   t=1;   
  } else
  if (strcasecmp(tmp.cell[1],"false")	== 0 ||
      strcasecmp(tmp.cell[1],"0")	== 0 ||
      strcasecmp(tmp.cell[1],"no")	== 0   )
  {
   t=0;
  } else
  {
   parse_error(name,line,tmp.cell[1]);
   t=-1;
  } 
 
 }

 return t;
}


int check_number(char *s, char *name, int line)
{
 int y;
 
 for (y=0; y<strlen(s); y++)
  if (isdigit(s[y])==0)
  {
   parse_error(name,line,s);
   return -1;
  }

 return 0;
}

int check_char(char *s, char *name, int line)
{
 if (strlen(s)>1)
 {
  fprintf(stderr,"line %d: invalid character %s\n",line+1,s);
  return -1;
 }
 return 0;
}



int check_color(s_line tmp, char *name, int line)
{
 int x,y,t;
 
 t=0; 
 
 if (tmp.cellcount!=4)
 { 
  if (tmp.cellcount>4)
   fprintf(stderr,"line %d: too many parameters\n",line+1);
  if (tmp.cellcount<4)
   fprintf(stderr,"line %d: too few parameters\n",line+1);
  t=-1;
 }
 
 for (x=1; x<4; x++)
 {
  for (y=0; y<strlen(tmp.cell[x]); y++)
  if (t==0 && isdigit(tmp.cell[x][y])==0)
  {
   parse_error(name,line,tmp.cell[x]);
   t=-1;
  }

  if (t==0 && (atol(tmp.cell[x])<0 || atol(tmp.cell[x])>255))
  {
   fprintf(stderr,"line %d: color value out of range %d\n",line,atol(tmp.cell[x]));
   t=-1;
  } 
 }

 return t;
}

void read_conf(char *name)
{
 int fd,mask=0;
 long line;
 s_line tmp;

/* Setting the defaults */ 
 conf_pinfo_fg		= CONF_DEF_PINFO_FG; 
 conf_pinfo_bg		= CONF_DEF_PINFO_BG;
 conf_arrow_fg		= CONF_DEF_ARROW_FG;
 conf_arrow_bg		= CONF_DEF_ARROW_BG;
 conf_mouse_color	= CONF_DEF_MOUSE_COLOR;
 conf_shirtlabel_fg	= CONF_DEF_SHIRTLABEL_FG;
 conf_shirtlabel_bg	= CONF_DEF_SHIRTLABEL_BG;
 conf_pantlabel_fg	= CONF_DEF_PANTLABEL_FG;
 conf_pantlabel_bg	= CONF_DEF_PANTLABEL_BG;
 conf_background	= CONF_DEF_BACKGROUND;
 conf_savebox_fg	= CONF_DEF_SAVEBOX_FG;
 conf_savebox_bg	= CONF_DEF_SAVEBOX_BG;
 shirtcolor		= CONF_DEF_SHIRT_COLOR;
 pantcolor		= CONF_DEF_PANT_COLOR;
 rounded		= CONF_DEF_CURSOR_ROUND;
 showrealc		= CONF_DEF_SHOW_REALC;
 conf_logo_red		= CONF_DEF_LOGO_RED;
 conf_logo_green	= CONF_DEF_LOGO_GREEN;
 conf_logo_blue		= CONF_DEF_LOGO_BLUE;

 key_brush_enlarge	= DEF_KEY_BRUSH_ENLARGE;
 key_brush_shrink	= DEF_KEY_BRUSH_SHRINK;
 key_brush_brighten	= DEF_KEY_BRUSH_BRIGHTEN;
 key_brush_darken	= DEF_KEY_BRUSH_DARKEN;
 key_brush_round	= DEF_KEY_BRUSH_ROUND;
 key_brush_c_next	= DEF_KEY_BRUSH_C_NEXT;
 key_brush_c_prev	= DEF_KEY_BRUSH_C_PREV;
 key_cursor_invert	= DEF_KEY_CURSOR_INVERT;
 key_mask_invert	= DEF_KEY_MASK_INVERT;
 key_draw_brighter	= DEF_KEY_DRAW_BRIGHTER;
 key_draw_darker	= DEF_KEY_DRAW_DARKER;
 key_show_realcolor	= DEF_KEY_SHOW_REALCOLOR;
 key_copy_set		= DEF_KEY_COPY_SET;
 key_copy_draw		= DEF_KEY_COPY_DRAW;
 key_save		= DEF_KEY_SAVE;
 key_quit		= DEF_KEY_QUIT;
 
 fd=open(name,O_RDONLY);
 if (fd==-1)
 {
  printf("%s not found, using defaults\n",name);
 } else
 {
  printf("reading configuration from %s\n",name);
  line=-1;
  tmp.eof=0;
  while (tmp.eof==0)
  {
   line++; tmp=s_rdln(fd);
   if (tmp.cellcount>0)
   {
    if (strcasecmp(tmp.cell[0],"pinfo_fg")==0)
     { 
      if (check_color(tmp,name,line)==0)
       conf_pinfo_fg=es_color(atol(tmp.cell[1]),
                              atol(tmp.cell[2]),
                              atol(tmp.cell[3]));
     } else
    if (strcasecmp(tmp.cell[0],"pinfo_bg")==0)
     { 
      if (check_color(tmp,name,line)==0)
       conf_pinfo_bg=es_color(atol(tmp.cell[1]),
                              atol(tmp.cell[2]),
                              atol(tmp.cell[3]));
     } else
    if (strcasecmp(tmp.cell[0],"arrow_fg")==0)
     { 
      if (check_color(tmp,name,line)==0)
       conf_arrow_fg=es_color(atol(tmp.cell[1]),
                              atol(tmp.cell[2]),
                              atol(tmp.cell[3]));
     } else
    if (strcasecmp(tmp.cell[0],"arrow_bg")==0)
     { 
      if (check_color(tmp,name,line)==0)
       conf_arrow_bg=es_color(atol(tmp.cell[1]),
                              atol(tmp.cell[2]),
                              atol(tmp.cell[3]));
     } else
    if (strcasecmp(tmp.cell[0],"mouse_color")==0)
     { 
      if (check_color(tmp,name,line)==0)
       conf_mouse_color=es_color(atol(tmp.cell[1]),
                                 atol(tmp.cell[2]),
                                 atol(tmp.cell[3]));
     } else
    if (strcasecmp(tmp.cell[0],"pantlabel_fg")==0)
     { 
      if (check_color(tmp,name,line)==0)
       conf_pantlabel_fg=es_color(atol(tmp.cell[1]),
                                  atol(tmp.cell[2]),
                                  atol(tmp.cell[3]));
     } else
    if (strcasecmp(tmp.cell[0],"pantlabel_bg")==0)
     { 
      if (check_color(tmp,name,line)==0)
       conf_pantlabel_bg=es_color(atol(tmp.cell[1]),
                                  atol(tmp.cell[2]),
                                  atol(tmp.cell[3]));
     } else
    if (strcasecmp(tmp.cell[0],"shirtlabel_fg")==0)
     { 
      if (check_color(tmp,name,line)==0)
       conf_shirtlabel_fg=es_color(atol(tmp.cell[1]),
                                   atol(tmp.cell[2]),
                                   atol(tmp.cell[3]));
     } else
    if (strcasecmp(tmp.cell[0],"shirtlabel_bg")==0)
     { 
      if (check_color(tmp,name,line)==0)
       conf_shirtlabel_bg=es_color(atol(tmp.cell[1]),
                                   atol(tmp.cell[2]),
                                   atol(tmp.cell[3]));
     } else
    if (strcasecmp(tmp.cell[0],"savebox_fg")==0)
     { 
      if (check_color(tmp,name,line)==0)
       conf_savebox_fg=es_color(atol(tmp.cell[1]),
                                atol(tmp.cell[2]),
                                atol(tmp.cell[3]));
     } else
    if (strcasecmp(tmp.cell[0],"savebox_bg")==0)
     { 
      if (check_color(tmp,name,line)==0)
       conf_savebox_bg=es_color(atol(tmp.cell[1]),
                                atol(tmp.cell[2]),
                                atol(tmp.cell[3]));
     } else
    if (strcasecmp(tmp.cell[0],"background")==0)
     { 
      if (check_color(tmp,name,line)==0)
       conf_background=es_color(atol(tmp.cell[1]),
                                atol(tmp.cell[2]),
                                atol(tmp.cell[3]));
     } else
    if (strcasecmp(tmp.cell[0],"shirt_color")==0)
     {
      if (tmp.cellcount<2)
        fprintf(stderr,"line %d: too few parameters\n",line+1);
       else
      if (tmp.cellcount>2)
        fprintf(stderr,"line %d: too many parameters\n",line+1);
       else
      if (check_number(tmp.cell[1],name,line)==0)
       if (atol(tmp.cell[1])>0 && atol(tmp.cell[1])<16)
        shirtcolor=atol(tmp.cell[1]);
       else
        fprintf(stderr,"line %d: invalid shirtcolor %d\n",line+1,atol(tmp.cell[1]));
     } else
    if (strcasecmp(tmp.cell[0],"pant_color")==0)
     {
      if (tmp.cellcount<2)
        fprintf(stderr,"line %d: too few parameters\n",line+1);
       else
      if (tmp.cellcount>2)
        fprintf(stderr,"line %d: too many parameters\n",line+1);
       else
      if (check_number(tmp.cell[1],name,line)==0)
       if (atol(tmp.cell[1])>0 && atol(tmp.cell[1])<16)
        pantcolor=atol(tmp.cell[1]);
       else
        fprintf(stderr,"line %d: invalid pantcolor %d\n",line+1,atol(tmp.cell[1]));
     } else
    if (strcasecmp(tmp.cell[0],"logo_color_red")==0)
     {
      if (tmp.cellcount<2)
        fprintf(stderr,"line %d: too few parameters\n",line+1);
       else
      if (tmp.cellcount>2)
        fprintf(stderr,"line %d: too many parameters\n",line+1);
       else
      if (check_number(tmp.cell[1],name,line)==0)
       if (atol(tmp.cell[1])>=0 && atol(tmp.cell[1])<101)
        conf_logo_blue=atol(tmp.cell[1]);
       else
        fprintf(stderr,"line %d: invalid logocolor %d\n",line+1,atol(tmp.cell[1]));
     } else
    if (strcasecmp(tmp.cell[0],"logo_color_green")==0)
     {
      if (tmp.cellcount<2)
        fprintf(stderr,"line %d: too few parameters\n",line+1);
       else
      if (tmp.cellcount>2)
        fprintf(stderr,"line %d: too many parameters\n",line+1);
       else
      if (check_number(tmp.cell[1],name,line)==0)
       if (atol(tmp.cell[1])>=0 && atol(tmp.cell[1])<101)
        conf_logo_green=atol(tmp.cell[1]);
       else
        fprintf(stderr,"line %d: invalid logocolor %d\n",line+1,atol(tmp.cell[1]));
     } else
    
    if (strcasecmp(tmp.cell[0],"logo_color_blue")==0)
     {
      if (tmp.cellcount<2)
        fprintf(stderr,"line %d: too few parameters\n",line+1);
       else
      if (tmp.cellcount>2)
        fprintf(stderr,"line %d: too many parameters\n",line+1);
       else
      if (check_number(tmp.cell[1],name,line)==0)
       if (atol(tmp.cell[1])>=0 && atol(tmp.cell[1])<101)
        conf_logo_blue=atol(tmp.cell[1]);
       else
        fprintf(stderr,"line %d: invalid logocolor %d\n",line+1,atol(tmp.cell[1]));
     } else
    if (strcasecmp(tmp.cell[0],"mask_file")==0)
     {
      if (tmp.cellcount==2)
      {
       conf_maskfile=(char *) malloc(strlen(tmp.cell[1]));
       memset(conf_maskfile,0,strlen(tmp.cell[1]));
       conf_maskfile=strcpy(conf_maskfile,tmp.cell[1]);
       mask=1;
      } else
       fprintf(stderr,"line %d: invalid number of parameters\n",line+1);
     } else
    if (strcasecmp(tmp.cell[0],"cursor_rounded")==0)
     { 
      if (check_boolean(tmp,name,line)!=-1)
       rounded=check_boolean(tmp,name,line);
     } else
    if (strcasecmp(tmp.cell[0],"show_realcolors")==0)
     { 
      if (check_boolean(tmp,name,line)!=-1)
       showrealc=check_boolean(tmp,name,line);
     } else
    if (strcasecmp(tmp.cell[0],"key_brush_enlarge")==0)
     {
      if (check_char(tmp.cell[1],name,line)!=-1)
       key_brush_enlarge=tmp.cell[1][0];
     } else
    if (strcasecmp(tmp.cell[0],"key_brush_shrink")==0)
     {
      if (check_char(tmp.cell[1],name,line)!=-1)
       key_brush_shrink=tmp.cell[1][0];
     } else
    if (strcasecmp(tmp.cell[0],"key_brush_brighten")==0)
     {
      if (check_char(tmp.cell[1],name,line)!=-1)
       key_brush_brighten=tmp.cell[1][0];
     } else
    if (strcasecmp(tmp.cell[0],"key_brush_darken")==0)
     {
      if (check_char(tmp.cell[1],name,line)!=-1)
       key_brush_darken=tmp.cell[1][0];
     } else
    if (strcasecmp(tmp.cell[0],"key_brush_round")==0)
     {
      if (check_char(tmp.cell[1],name,line)!=-1)
       key_brush_round=tmp.cell[1][0];
     } else
    if (strcasecmp(tmp.cell[0],"key_brush_c_next")==0)
     {
      if (check_char(tmp.cell[1],name,line)!=-1)
       key_brush_c_next=tmp.cell[1][0];
     } else
    if (strcasecmp(tmp.cell[0],"key_brush_c_prev")==0)
     {
      if (check_char(tmp.cell[1],name,line)!=-1)
       key_brush_c_prev=tmp.cell[1][0];
     } else
    if (strcasecmp(tmp.cell[0],"key_cursor_invert")==0)
     {
      if (check_char(tmp.cell[1],name,line)!=-1)
       key_cursor_invert=tmp.cell[1][0];
     } else
    if (strcasecmp(tmp.cell[0],"key_mask_invert")==0)
     {
      if (check_char(tmp.cell[1],name,line)!=-1)
       key_mask_invert=tmp.cell[1][0];
     } else
    if (strcasecmp(tmp.cell[0],"key_draw_brighter")==0)
     {
      if (check_char(tmp.cell[1],name,line)!=-1)
       key_draw_brighter=tmp.cell[1][0];
     } else
    if (strcasecmp(tmp.cell[0],"key_draw_darker")==0)
     {
      if (check_char(tmp.cell[1],name,line)!=-1)
       key_draw_darker=tmp.cell[1][0];
     } else
    if (strcasecmp(tmp.cell[0],"key_show_realcolor")==0)
     {
      if (check_char(tmp.cell[1],name,line)!=-1)
       key_show_realcolor=tmp.cell[1][0];
     } else
    if (strcasecmp(tmp.cell[0],"key_copy_set")==0)
     {
      if (check_char(tmp.cell[1],name,line)!=-1)
       key_copy_set=tmp.cell[1][0];
     } else
    if (strcasecmp(tmp.cell[0],"key_copy_draw")==0)
     {
      if (check_char(tmp.cell[1],name,line)!=-1)
       key_copy_draw=tmp.cell[1][0];
     } else
    if (strcasecmp(tmp.cell[0],"key_save")==0)
     {
      if (check_char(tmp.cell[1],name,line)!=-1)
       key_save=tmp.cell[1][0];
     } else
    if (strcasecmp(tmp.cell[0],"key_quit")==0)
     {
      if (check_char(tmp.cell[1],name,line)!=-1)
       key_quit=tmp.cell[1][0];
     } else
    parse_error(name,line,tmp.cell[0]);
   }
  
  
  }   

  close(fd);
 }

 if (mask==0)
 {
  conf_maskfile=(char *) malloc(strlen(CONF_DEF_MASK_FILE));
  memset(conf_maskfile,0,strlen(CONF_DEF_MASK_FILE));
  conf_maskfile=strcpy(conf_maskfile,CONF_DEF_MASK_FILE);
 }

}
